// code permettant de lire la température et l'humidité avec un capteur DHT22 branché sur le pin 7 et
// la luminosité avec une photorésistance branchée sur le pin A1 (faire un pont avec une résistance 1kOhm).
// Une LED est branchée sur le pin 13 et s'allume à chaque mesure.
// Mise en veille à régler en heures/minutes

#include <SPI.h>
#include <SD.h>
#include <DHT.h>
#include <RTCZero.h>

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

const int chipSelect = 4; // SD pin 

#define DHTPIN 7 // DHT pin and type
#define DHTTYPE DHT22
DHT dht(DHTPIN, DHTTYPE);

const int ledPin = 13; // Led pin
const int ldrPin = A1; // LDR pin

RTCZero rtc; // real-time clock

bool matched = true; // alarm matched

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

void setup() {
  
  Serial.begin(9600); // serial communication

  delay(1000);
  
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  Serial.print("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present.");
    // don't do anything more:
  while (1);
  }
  
  Serial.println("Card initialized.");

  delay(1000);

  dht.begin(); // DHT initialization

  delay(1000);
  
  pinMode(ledPin, OUTPUT); // Led def
  pinMode(ldrPin, INPUT); // LDR def
  
  Serial.println("DHT and LDR initialized.");

  rtc.begin(); // initialize real-time clock
  
  setTimeFromCompile(); // get time and date from the compiler
  setDateFromCompile();

  Serial.println("Clock initialized.");  

  File dataFile = SD.open("datalog.txt", FILE_WRITE); // open the datafile

  String titleString = ""; 

  titleString = String("Time") + "\t" + String("Temperature(C)") + "\t" + String("Humidity(%)") + "\t" + String("Light(V)");

  // if the file is available, write to it:
  if (dataFile) {
   dataFile.println(titleString);
   dataFile.close();
   // print to the serial port too:
   //Serial.println(titleString);
   }
   // if the file isn't open, pop up an error:
   else {
    Serial.println("Error opening datalog.txt");
  }
}

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

void loop() {

  if (matched){

    matched = false;
  
    digitalWrite(ledPin, HIGH); // Switch the led on

    Serial.println("Starting experiment!");

    float Temp = dht.readTemperature(); // Read the sensors
    float Humidity = dht.readHumidity();
    float Light = analogRead(ldrPin);

    int hours = rtc.getHours();
    int minutes = rtc.getMinutes(); 
 
    String dataString = ""; 

    dataString += String(hours) + String(":") + String(minutes)+ "\t" + String(Temp) + "\t" + String(Humidity) + "\t" + String(Light);  
  
    File dataFile = SD.open("datalog.txt", FILE_WRITE);

    // if the file is available, write to it:
    if (dataFile) {
      dataFile.println(dataString);
      dataFile.close();
      // print to the serial port too:
      //Serial.println(dataString);
    }
    // if the file isn't open, pop up an error:
    //else {
      //Serial.println("Error opening datalog.txt");
    //}

    digitalWrite(ledPin, LOW); // switch the led off
  
    delay(1000); 

    //int alarmMinutes = rtc.getMinutes();
    int alarmHours = rtc.getHours();
    
    //alarmMinutes += 30; // set alarm every 30mn
    alarmHours += 3; // set alarm every 3h
    
    //if (alarmMinutes > 59) {
      //alarmMinutes -= 60;
      //alarmHours += 1;
    //}

    if(alarmHours > 23) {
      alarmHours -= 24;     
    }

    rtc.setAlarmTime(alarmHours, 00, 00);
    rtc.enableAlarm(rtc.MATCH_HHMMSS);
    rtc.attachInterrupt(alarmMatch);
    rtc.standbyMode();    // Sleep until next alarm match    
  }    
}

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

// set the rtc time from the compile time:
void setTimeFromCompile() {
  // get the compile time string:
  String compileTime = String(__TIME__);
  // break the compile time on the colons:
  int h = compileTime.substring(0, 2).toInt();
  int m = compileTime.substring(3, 5).toInt();
  int s = compileTime.substring(6, 8).toInt();

  // set the time from the derived numbers:
  rtc.setTime(h, m, s);
}

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

// set the rtc time from the compile date:
void setDateFromCompile() {
  String months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };
  // get the compile date:
  String compileDate = String(__DATE__);

  // get the date substring
  String monthStr = compileDate.substring(0, 3);

  int m = 0;    // variable for the date as an integer
  // see which month matches the month string:
  for (int i = 0; i < 12; i++) {
    if (monthStr == months[i]) {
      // the first month is 1, but its array position is 0, so:
      m = i + 1;
      // no need to continue the rest of the for loop:
      break;
    }
  }

  // get the day and year as substrings, and convert to numbers:
  int d = compileDate.substring(4, 6).toInt();
  int y = compileDate.substring(9, 11).toInt();
  // set the date from the derived numbers:
  rtc.setDate(d, m, y);
}

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

void alarmMatch(){
  matched=true;
}
